/** @file sys_main.c 
*   @brief Application main file
*   @date 17.Nov.2014
*   @version 04.02.00
*
*   This file contains an empty main function,
*   which can be used for the application.
*/

/* 
* Copyright (C) 2009-2014 Texas Instruments Incorporated - http://www.ti.com/ 
* 
* 
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*
*    Redistributions of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


/* USER CODE BEGIN (0) */
/* USER CODE END */

/* Include Files */

#include "sys_common.h"

/* USER CODE BEGIN (1) */
#include "sys_vim.h"
#include "sci.h"

#include "usblib.h"
#include "usbcdc.h"
#include "usbdcdc.h"
#include "usb_serial_structs.h"
/* USER CODE END */

/** @fn void main(void)
*   @brief Application main function
*   @note This function is empty by default.
*
*   This function is called after startup.
*   The user can use this function to implement the application.
*/

/* USER CODE BEGIN (2) */
uint8		txtCRLF[]			= {'\r', '\n'};
uint8  	txtTitle[] 			= {"HERCULES MICROCONTROLLERS"};
uint8		txtTI[]				= {"    Texas Instruments    "};
uint8		txtLittleEndian[] 	= {"   Little Endian device  "};
uint8		txtBigEndian[]		= {"    Big Endian device    "};
//*****************************************************************************
//
// Global flag indicating that a USB configuration has been set.
//
//*****************************************************************************
static volatile tBoolean g_bUSBConfigured = FALSE;

//*****************************************************************************
//
// Setting the structure to line coding
//
//*****************************************************************************
#define DEFAULT_BIT_RATE 115200
static tLineCoding defaultLineCoding = {DEFAULT_BIT_RATE, USB_CDC_STOP_BITS_1, USB_CDC_PARITY_NONE, 8};

//*****************************************************************************
//
// Function definitions
//
//*****************************************************************************
		void		sciDisplayText	(sciBASE_t *sci, uint8 *text,uint32 length);
static	void 		IntEnable		(void);
		uint8 	IntDisable		(void);
		void 		IntRestore		(uint8  status);
		uint32	RxHandler		(void *pvCBData, uint32 ulEvent, uint32 ulMsgValue, void *pvMsgData);
		uint32	TxHandler		(void *pvCBData, uint32 ulEvent, uint32 ulMsgValue, void *pvMsgData);
		uint32 	ControlHandler	(void *pvCBData, uint32 ulEvent, uint32 ulMsgValue, void *pvMsgData);
static 	void 		GetLineCoding	(tLineCoding *psLineCoding);
static 	tBoolean 	SetLineCoding	(tLineCoding *psLineCoding);

extern void 		sciDisplayText	(sciBASE_t *sci, uint8 *text,uint32 length);
extern void		USBDeviceIntHandlerInternal	(uint32 uIndex, uint32 uStatus);

/* USER CODE END */

void main(void)
{
/* USER CODE BEGIN (3) */
	uint8 ucBuffer[64];
	volatile uint32 ulDataWrote = 0,  ulTemp = 0;


	/* Enable the interrupts in CPSR register */
	IntEnable();
	/* Enable the SCI for console output */
	sciInit();

	/* Dispaly welcome message */
	sciDisplayText(scilinREG, txtCRLF, sizeof(txtCRLF));
	sciDisplayText(scilinREG, txtTI, sizeof(txtTI));
	sciDisplayText(scilinREG, txtCRLF, sizeof(txtCRLF));
	sciDisplayText(scilinREG, txtTitle, sizeof(txtTitle));
	sciDisplayText(scilinREG, txtCRLF, sizeof(txtCRLF));

	#ifdef __little_endian__
	    sciDisplayText(scilinREG, txtLittleEndian, sizeof(txtLittleEndian));
	    sciDisplayText(scilinREG, txtCRLF, sizeof(txtCRLF));
	#else
	    sciDisplayText(scilinREG, txtBigEndian, sizeof(txtBigEndian));
	    sciDisplayText(scilinREG, txtCRLF, sizeof(txtCRLF));
	#endif

		//
	//Initialize the USB Rx and TX Buffers
	//
	USBBufferInit((tUSBBuffer *)&g_sTxBuffer);
	USBBufferInit((tUSBBuffer *)&g_sRxBuffer);

	//
	// Pass our device information to the USB library and place the device
	// on the bus.
	//
	USBDCDCInit(0, (tUSBDCDCDevice *)&g_sCDCDevice);

	//
	// See if we have rcvd any data. Echo any rcvd data
	//
	while(1) {
		if (USBBufferDataAvailable((tUSBBuffer *)&g_sRxBuffer) > 0) {
			do {
				ulTemp = USBBufferRead((tUSBBuffer *)&g_sRxBuffer, ucBuffer, 1);
				if (ulTemp > 0) {
					USBBufferWrite((tUSBBuffer *)&g_sTxBuffer, ucBuffer, 1);
				}
			} while (ulTemp > 0);
		}
	}
/* USER CODE END */
}

/* USER CODE BEGIN (4) */
/**
 * \brief  Enable the IRQ (CPSR I bit)
 *
 * \param    void
 *
 * \return   None
 *
 *  Note: This function call shall be done only in previleged mode of ARM
 **/
static void IntEnable(void)
{
	_enable_IRQ();
	return;
}


/**
 * \brief Read and save the stasus and Disables the processor IRQ .
 *         Prevents the processor to respond to IRQs.
 *
 *
 * \return   Current status of IRQ
 *
 *  Note: This function call shall be done only in previleged mode of ARM
 **/
uint8 IntDisable(void)
{
	uint8 status = 0;

	/* Reads the current status.*/
	status = 0x80 & _get_CPSR();

	/* Disable the Interrupts.*/
	_disable_IRQ();

	return status;
}


/**
 * \brief  Restore the processor IRQ only status. This does not affect
 *          the set of interrupts enabled/disabled in the AINTC.
 *
 * \param  status - The status returned by the IntDisable fundtion.
 *
 * \return   None
 *
 *  Note: This function call shall be done only in previleged mode of ARM
 **/
void IntRestore(uint8  status)
{
	if (status & 0x80) {
		/* CPSR INT bit was already disabled, so leave it */
	} else {
		_enable_IRQ();
	}
	return;
}

//*****************************************************************************
//
// Handles CDC driver notifications related to the receive channel (data from
// the USB host).
//
// \param ulCBData is the client-supplied callback data value for this channel.
// \param ulEvent identifies the event we are being notified about.
// \param ulMsgValue is an event-specific value.
// \param pvMsgData is an event-specific pointer.
//
// This function is called by the CDC driver to notify us of any events
// related to operation of the receive data channel (the OUT channel carrying
// data from the USB host).
//
// \return The return value is event-specific.
//
//*****************************************************************************
uint32 RxHandler(void *pvCBData, uint32 ulEvent, uint32 ulMsgValue, void *pvMsgData)
{
    //
    // Which event are we being sent?
    //
    switch(ulEvent)
    {
        //
        // A new packet has been received. Update the variable so that the non-interrupt (main) can
        //
        case USB_EVENT_RX_AVAILABLE:
        {
        	break;
        }

        //
        // We are being asked how much unprocessed data we have still to
        // process. We return 0 if the UART is currently idle or 1 if it is
        // in the process of transmitting something. The actual number of
        // bytes in the UART FIFO is not important here, merely whether or
        // not everything previously sent to us has been transmitted.
        //
        case USB_EVENT_DATA_REMAINING:
        {
            break;
        }

        //
        // We are being asked to provide a buffer into which the next packet
        // can be read. We do not support this mode of receiving data so let
        // the driver know by returning 0. The CDC driver should not be sending
        // this message but this is included just for illustration and
        // completeness.
        //
        case USB_EVENT_REQUEST_BUFFER:
        {
            break;
        }

        //
        // We don't expect to receive any other events.  Ignore any that show
        // up in a release build or hang in a debug build.
        //
        default:
#ifdef DEBUG
            while(1);
#else
            break;
#endif
    }

    return(0);
}


//*****************************************************************************
//
// Handles CDC driver notifications related to control and setup of the device.
//
// \param pvCBData is the client-supplied callback pointer for this channel.
// \param ulEvent identifies the event we are being notified about.
// \param ulMsgValue is an event-specific value.
// \param pvMsgData is an event-specific pointer.
//
// This function is called by the CDC driver to perform control-related
// operations on behalf of the USB host.  These functions include setting
// and querying the serial communication parameters, setting handshake line
// states and sending break conditions.
//
// \return The return value is event-specific.
//
//*****************************************************************************
uint32
ControlHandler(void *pvCBData, uint32 ulEvent,
               uint32 ulMsgValue, void *pvMsgData)
{
    //
    // Which event are we being asked to process?
    //
    switch(ulEvent)
    {
        //
        // We are connected to a host and communication is now possible.
        //
        case USB_EVENT_CONNECTED:
            g_bUSBConfigured = TRUE;

            //
            // Flush our buffers.
            //
            USBBufferFlush(&g_sTxBuffer);
            USBBufferFlush(&g_sRxBuffer);
            break;

        //
        // The host has disconnected.
        //
        case USB_EVENT_DISCONNECTED:
            g_bUSBConfigured = FALSE;
            break;

        //
        // Return the current serial communication parameters.
        //
        case USBD_CDC_EVENT_GET_LINE_CODING:
            GetLineCoding(pvMsgData);
            break;

        //
        // Set the current serial communication parameters.
        //
        case USBD_CDC_EVENT_SET_LINE_CODING:
            SetLineCoding(pvMsgData);
            break;

        //
        // Set the current serial communication parameters.
        //
        case USBD_CDC_EVENT_SET_CONTROL_LINE_STATE:
            break;

        //
        // Send a break condition on the serial line.
        //
        case USBD_CDC_EVENT_SEND_BREAK:
            break;

        //
        // Clear the break condition on the serial line.
        //
        case USBD_CDC_EVENT_CLEAR_BREAK:
            break;

        //
        // Ignore SUSPEND and RESUME for now.
        //
        case USB_EVENT_SUSPEND:
        case USB_EVENT_RESUME:
            break;

        //
        // We don't expect to receive any other events.  Ignore any that show
        // up in a release build or hang in a debug build.
        //
        default:
#ifdef DEBUG
            while(1);
#else
            break;
#endif

    }

    return(0);
}


//*****************************************************************************
//
// Handles CDC driver notifications related to the transmit channel (data to
// the USB host).
//
// \param ulCBData is the client-supplied callback pointer for this channel.
// \param ulEvent identifies the event we are being notified about.
// \param ulMsgValue is an event-specific value.
// \param pvMsgData is an event-specific pointer.
//
// This function is called by the CDC driver to notify us of any events
// related to operation of the transmit data channel (the IN channel carrying
// data to the USB host).
//
// \return The return value is event-specific.
//
//*****************************************************************************
uint32 TxHandler(void *pvCBData, uint32 ulEvent, uint32 ulMsgValue, void *pvMsgData)
{
    //
    // Which event have we been sent?
    //
    switch(ulEvent)
    {
        case USB_EVENT_TX_COMPLETE:
            //
            // Since we are using the USBBuffer, we don't need to do anything
            // here.
            //
            break;

        //
        // We don't expect to receive any other events.  Ignore any that show
        // up in a release build or hang in a debug build.
        //
        default:
#ifdef DEBUG
            while(1);
#else
            break;
#endif

    }
    return(0);
}


//*****************************************************************************
//
// Get the communication parameters in use on the UART.
//
//*****************************************************************************
static void
GetLineCoding(tLineCoding *psLineCoding)
{
    //
    // Get the current line coding set in the UART.
    //
    psLineCoding->ulRate = defaultLineCoding.ulRate;
    psLineCoding->ucDatabits = defaultLineCoding.ucDatabits;
    psLineCoding->ucParity = defaultLineCoding.ucParity;
    psLineCoding->ucStop = defaultLineCoding.ucStop;

}

//*****************************************************************************
//
// Set the communication parameters to use on the UART.
//
//*****************************************************************************
static tBoolean SetLineCoding(tLineCoding *psLineCoding)
{
    tBoolean bRetcode = TRUE;

	defaultLineCoding.ulRate = psLineCoding->ulRate;
	defaultLineCoding.ucStop = psLineCoding->ucStop;
	defaultLineCoding.ucParity = psLineCoding->ucParity;
	defaultLineCoding.ucDatabits = psLineCoding->ucDatabits;

	//
    // Let the caller know if we had a problem or not.
    //
    return(bRetcode);
}

#pragma INTERRUPT(serviceIrq, IRQ)
void serviceIrq(void)
{
	USBDeviceIntHandlerInternal(0, 0);
	return;
}
/* Function to display test on JTAG USB Virtual COM Port (SCI2) */
void sciDisplayText(sciBASE_t *sci, uint8 *text,uint32 length)
{
    while(length--)
    {
        while ((scilinREG->FLR & 0x4) == 4); /* wait until busy */
        sciSendByte(scilinREG,*text++);      /* send out text   */
    };
}

/* USER CODE END */
